@@ -142,12 +142,12 @@ DEFAULT_CONFIGURATION = {
     u'support_negative_year': True,  # snowflake
     u'log_max_query_length': LOG_MAX_QUERY_LENGTH,  # snowflake
     u'disable_request_pooling': False,  # snowflake
-    # Enable temporary credential file for Linux, default false. Mac/Win will overlook this
-    u'client_store_temporary_credential': False,
+    u'client_store_temporary_credential': False,  # enable temporary credential file for Linux, default false. Mac/Win will overlook this
     'use_openssl_only': False,  # only use openssl instead of python only crypto modules
 }
 
-APPLICATION_RE = re.compile(r'^[\w.-]+$')
+APPLICATION_RE = re.compile(r'[\w\d_]+')
+
 # adding the exception class to Connection class
 for m in [method for method in dir(errors) if
           callable(getattr(errors, method))]:
@@ -159,21 +159,6 @@ strptime('20150102030405', '%Y%m%d%H%M%S')
 logger = getLogger(__name__)
 
 
-def verify_application(val: str) -> None:
-    """Raise ProgrammingError if invalid application name"""
-    if len(val) > 50:
-        raise ProgrammingError(
-            msg="Application name is too long: {}".format(val),
-            errno=0
-        )
-
-    if not APPLICATION_RE.match(val):
-        raise ProgrammingError(
-            msg='Invalid application name: {}'.format(val),
-            errno=0
-        )
-
-
 class SnowflakeConnection(object):
     u"""
     Implementation of the connection object for the Snowflake Database. Use
@@ -417,10 +402,6 @@ class SnowflakeConnection(object):
         """
         return self._application
 
-    @application.setter
-    def application(self, val: str) -> None:
-        verify_application(val)
-
     @property
     def errorhandler(self):
         u"""
@@ -720,7 +701,7 @@ class SnowflakeConnection(object):
             # enable storing temporary credential in a file
             self._session_parameters[
                 PARAMETER_CLIENT_STORE_TEMPORARY_CREDENTIAL] = \
-                self._client_store_temporary_credential if IS_LINUX else True
+                    self._client_store_temporary_credential if IS_LINUX else True
 
         auth = Auth(self.rest)
         if not auth.read_temporary_credential(
@@ -746,10 +727,16 @@ class SnowflakeConnection(object):
             if name == u'sequence_counter':
                 self.sequence_counter = value
             elif name == u'application':
-                verify_application(value)
-                setattr(self, '_' + name, value)
+                if not APPLICATION_RE.match(value):
+                    msg = u'Invalid application name: {}'.format(value)
+                    raise ProgrammingError(
+                        msg=msg,
+                        errno=0
+                    )
+                else:
+                    setattr(self, u'_' + name, value)
             else:
-                setattr(self, '_' + name, value)
+                setattr(self, u'_' + name, value)
 
         if self._numpy:
             try:
