@@ -1,27 +1,27 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2018 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2019 Snowflake Computing Inc. All right reserved.
 #
 from datetime import timedelta
 from logging import getLogger
 
 import pytz
 
-from .converter import (SnowflakeConverter, ZERO_EPOCH)
+from .converter import ZERO_EPOCH, SnowflakeConverter, _generate_tzinfo_from_tzoffset
 
 logger = getLogger(__name__)
 
 
 class SnowflakeConverterIssue23517(SnowflakeConverter):
     """
-    Converter for Python 3.4.3 and 3.5.0
+    Converter for Python 3.5.0 or Any Python on Windows
     This is to address http://bugs.python.org/issue23517
     """
 
     def __init__(self, **kwargs):
         super(SnowflakeConverterIssue23517, self).__init__(**kwargs)
-        logger.info('initialized')
+        logger.debug('initialized')
 
     def _TIMESTAMP_TZ_to_python(self, ctx):
         """
@@ -34,22 +34,20 @@ class SnowflakeConverterIssue23517(SnowflakeConverter):
 
         def conv0(encoded_value):
             value, tz = encoded_value.split()
-            tzinfo = SnowflakeConverter._generate_tzinfo_from_tzoffset(
-                int(tz) - 1440)
+            tzinfo = _generate_tzinfo_from_tzoffset(int(tz) - 1440)
             microseconds = float(value)
             t = ZERO_EPOCH + timedelta(seconds=microseconds)
             if pytz.utc != tzinfo:
-                t += tzinfo.utcoffset(t, is_dst=False)
+                t += tzinfo.utcoffset(t)
             return t.replace(tzinfo=tzinfo)
 
         def conv(encoded_value):
             value, tz = encoded_value.split()
-            tzinfo = SnowflakeConverter._generate_tzinfo_from_tzoffset(
-                int(tz) - 1440)
+            tzinfo = _generate_tzinfo_from_tzoffset(int(tz) - 1440)
             microseconds = float(value[0:-scale + 6])
             t = ZERO_EPOCH + timedelta(seconds=microseconds)
             if pytz.utc != tzinfo:
-                t += tzinfo.utcoffset(t, is_dst=False)
+                t += tzinfo.utcoffset(t)
             return t.replace(tzinfo=tzinfo)
 
         return conv if scale > 6 else conv0
@@ -91,7 +89,7 @@ class SnowflakeConverterIssue23517(SnowflakeConverter):
         scale = ctx['scale']
 
         conv0 = lambda value: (
-            ZERO_EPOCH + timedelta(seconds=(float(value)))).time()
+                ZERO_EPOCH + timedelta(seconds=(float(value)))).time()
 
         def conv(value):
             microseconds = float(value[0:-scale + 6])
